<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;

use Util\DateUtil;
use Util\NovoCodigo;
use Ems\Util\Utils;
use Ems\Services\MarcacaoService;

// MENSAGENS
define('messagesMarcacaoRepository', [
    'ERRO_VINCULO_NAO_ENCONTRADO'  => 'Vínculo não foi encontrado!',
    'ERRO_PARAMORG_NAO_ENCONTRADO' => 'Parâmetros da estrutura não encontrado!'
]);

class MarcacaoRepository {
    protected $connection;

    const TIPO_VINC_CLT = 1;
    const VINC_EMPREG_APRENDIZ = 9;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getMarcacao(int $nrOrg, int $nrMarcacao) {
        $params = [
            'P_NRORG'      => $nrOrg,
            'P_NRMARCACAO' => $nrMarcacao
        ];

        $sql = <<<SQL
            SELECT GPE_MARCACAOPONTO.*,
                   TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY') DIA
            FROM GPE_MARCACAOPONTO
            WHERE NRORG = :P_NRORG
              AND NRMARCACAOPONTO = :P_NRMARCACAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacao = $std->fetch();

        return $marcacao;
    }

    public function getVinculosGestor($params) {
        $sql = <<<SQL
            SELECT
                gpe_vinculom.nrvinculom,
                gpe_vinculom.nmvinculom,
                gpe_vinculom.dtadmissaovinc,
                gpe_vinculom.dtrescisaovinc,
                gpe_vinculom.nrtipovinculom,
                gpe_vinculom.nrpessoa
            FROM
                gpe_vinculom
                JOIN gpe_vinculoh ON gpe_vinculoh.nrorg = gpe_vinculom.nrorg
                                    AND gpe_vinculoh.nrvinculom = gpe_vinculom.nrvinculom
                                    AND gpe_vinculoh.dtmescompetenc = (
                    SELECT
                        MAX(dtmescompetenc)
                    FROM
                        gpe_vinculoh h1
                    WHERE
                        h1.nrorg = gpe_vinculoh.nrorg
                        AND h1.nrvinculom = gpe_vinculoh.nrvinculom
                        AND h1.dtmescompetenc <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                )
                JOIN estruturah ON estruturah.nrorg = gpe_vinculoh.nrorg
                                AND estruturah.nrestruturam = gpe_vinculoh.nrestrutgeren
                                AND ESTRUTURAH.dtmescompetenc = (
                    SELECT
                        MAX(dtmescompetenc)
                    FROM ESTRUTURAH h1
                    WHERE
                        h1.nrorg = estruturah.nrorg
                        AND h1.nrestruturam = estruturah.nrestruturam
                        AND h1.dtmescompetenc <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                )
            WHERE
                gpe_vinculom.nrorg = :P_NRORG
                AND (INSTR(:P_NRESTRUTURAS, ',' || ESTRUTURAH.NRESTRUTURAM || ',')) > 0
                AND ( gpe_vinculom.dtrescisaovinc IS NULL
                    OR gpe_vinculom.dtrescisaovinc >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') )
            ORDER BY
                gpe_vinculom.nmvinculom
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /*
    * Função pra poder montar o novo regitro de marcação que será inserido
    */
    public function getHorarioBase(
        string $nrvinculom,
        string $nrorg,
        string $date = null
    ) : array {
        if ($date === null) {
            $date = new \DateTime();
            $date = $date->format('d/m/Y');
        }

        try {
            $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrvinculom, $nrorg, $date);
            $nrhordiam = $dadosHorarioVinculo['NRHORDIAM'];

            $paramorg = $this->getParamorg($nrorg);

            if (empty($paramorg)) {
                throw new \Exception(constant('messagesMarcacaoRepository')['ERRO_PARAMORG_NAO_ENCONTRADO']);
            }

            $toleranciaAnterior  = DateUtil::getHoraFloatDeString($paramorg['HRTEMPOTOLERANCIAANT']);
            $toleranciaPosterior = DateUtil::getHoraFloatDeString($paramorg['HRTEMPOTOLERANCIAPOS']);


            $sql = <<<SQL

                SELECT
                    --TO_CHAR(TO_DATE(HRENTRADA, 'HH24MI') - 0.003472, 'HH24:MI') AS HRENTRADA_NOVA,
                    SUBSTR(SUBSTR(HRENTRADA, 1, LENGTH(TRIM(HRENTRADA)) -2) || ':' || SUBSTR(HRENTRADA, -2), 1, 10) AS HRENTRADA,
                    SUBSTR(SUBSTR(HRPROJETARINTERVALO, 1, LENGTH(TRIM(HRPROJETARINTERVALO)) -2) || ':' || SUBSTR(HRPROJETARINTERVALO, -2), 1, 10) AS HRPROJETARINTERVALO,
                    (SELECT SUBSTR(SUBSTR(HRSAIDA, 1, LENGTH(TRIM(HRSAIDA)) -2) || ':' || SUBSTR(HRSAIDA, -2), 1, 10) AS HRSAIDA
                        FROM GPE_INTERVALOSHORARIO IH
                    WHERE IH.NRORG     = :P_NRORG
                        AND IH.NRHORDIAM = :P_NRHORDIAM
                        AND IH.NRSEQINTERVHORARIO = (SELECT MAX(NRSEQINTERVHORARIO)
                                                            FROM GPE_INTERVALOSHORARIO
                                                        WHERE NRORG     = IH.NRORG
                                                            AND NRHORDIAM = IH.NRHORDIAM)) HRSAIDA
                FROM GPE_INTERVALOSHORARIO IH

                WHERE IH.NRORG     = :P_NRORG
                    AND IH.NRHORDIAM = :P_NRHORDIAM
                    AND IH.NRSEQINTERVHORARIO = (SELECT MIN(NRSEQINTERVHORARIO)
                                                    FROM GPE_INTERVALOSHORARIO
                                                    WHERE NRORG     = IH.NRORG
                                                    AND NRHORDIAM = IH.NRHORDIAM)

SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"     => $nrorg,
                    "P_NRHORDIAM" => $nrhordiam
                )
            );

            $horariobase = $query->fetchAll();

            if (count($horariobase) > 0) {
                $horariobase[0]["HRENTRADA_NOVA"] = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRENTRADA"]) - $toleranciaAnterior, true);
                $horariobase[0]["HRSAIDA_NOVA"]   = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRSAIDA"]) + $toleranciaPosterior, true);

                return $horariobase[0];
            } else {
                throw new \Exception("Horário base não encontrado.", 1);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereMarcacoesPontoManual(
        string $nrvinculom,
        string $dthrmarcacaoponto,
        string $dthrajustada,
        string $idenquadrahorario,
        string $idmarcdesprezada,
        string $iddiaocorrencia,
               $dsjustificativa,
        string $nrorginclusao,
        int    $nrcoletor,
        string $idreferintervalo,
               $cdcnpjestrut,
               $nmrazsocestrut,
               $cdcaepfestrut,
               $cdceiestrut,
               $cdticketaut,
               $dslatitude,
               $dslongitude,
               $dspathfoto,
        string $idativo,
        string $cdoperinclusao,
        string $nrorg,
               $nrMarcacao = null
    ) : string {

        $nrmarcacaoponto =
            $nrMarcacao ??
            NovoCodigo::geraCodigo($nrorg, 'GPE_MARCACAOPONTO', 1, 22, false);

        $dtinclusao = $dthrmarcacaoponto;

        $sql = <<<SQL
            INSERT INTO GPE_MARCACAOPONTO (
                NRMARCACAOPONTO,
                NRVINCULOM,
                DTHRMARCACAOPONTO,
                DTHRAJUSTADA,
                IDENQUADRAHORARIO,
                IDMARCDESPREZADA,
                IDDIAOCORRENCIA,
                DSJUSTIFICATIVA,
                NRORGINCLUSAO,
                NRCOLETOR,
                IDREFERINTERVALO,
                CDCNPJESTRUT,
                NMRAZSOCESTRUT,
                CDCAEPFESTRUT,
                CDCEIESTRUT,
                CDTICKETAUT,
                DSLATITUDE,
                DSLONGITUDE,
                DSPATHFOTO,
                IDATIVO,
                DTINCLUSAO,
                CDOPERINCLUSAO,
                NRORG
            ) VALUES (
                :P_NRMARCACAOPONTO,
                :P_NRVINCULOM,
                TO_DATE(:P_DTHRMARCACAOPONTO, 'DD/MM/YYYY HH24:MI:SS'),
                TO_DATE(:P_DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS'),
                :P_IDENQUADRAHORARIO,
                :P_IDMARCDESPREZADA,
                :P_IDDIAOCORRENCIA,
                :P_DSJUSTIFICATIVA,
                :P_NRORGINCLUSAO,
                :P_NRCOLETOR,
                :P_IDREFERINTERVALO,
                :P_CDCNPJESTRUT,
                :P_NMRAZSOCESTRUT,
                :P_CDCAEPFESTRUT,
                :P_CDCEIESTRUT,
                :P_CDTICKETAUT,
                :P_DSLATITUDE,
                :P_DSLONGITUDE,
                :P_DSPATHFOTO,
                :P_IDATIVO,
                SYSDATE,
                :P_CDOPERINCLUSAO,
                :P_NRORG
            )
SQL;

        $query = $this->connection->prepare($sql);

        $result = $query->execute(
            array(
                "P_NRMARCACAOPONTO"     => $nrmarcacaoponto,
                "P_NRVINCULOM"          => $nrvinculom,
                "P_DTHRMARCACAOPONTO"   => $dthrmarcacaoponto,
                "P_DTHRAJUSTADA"        => $dthrajustada,
                "P_IDENQUADRAHORARIO"   => $idenquadrahorario,
                "P_IDMARCDESPREZADA"    => $idmarcdesprezada,
                "P_IDDIAOCORRENCIA"     => $iddiaocorrencia,
                "P_DSJUSTIFICATIVA"     => $dsjustificativa,
                "P_NRORGINCLUSAO"       => $nrorginclusao,
                "P_NRCOLETOR"           => $nrcoletor,
                "P_IDREFERINTERVALO"    => $idreferintervalo,
                "P_CDCNPJESTRUT"        => $cdcnpjestrut,
                "P_NMRAZSOCESTRUT"      => $nmrazsocestrut,
                "P_CDCAEPFESTRUT"       => $cdcaepfestrut,
                "P_CDCEIESTRUT"         => $cdceiestrut,
                "P_CDTICKETAUT"         => $cdticketaut,
                "P_DSLATITUDE"          => $dslatitude,
                "P_DSLONGITUDE"         => $dslongitude,
                "P_DSPATHFOTO"          => $dspathfoto,
                "P_IDATIVO"             => $idativo,
                "P_CDOPERINCLUSAO"      => $cdoperinclusao,
                "P_NRORG"               => $nrorg
            )
        );

        if ($result) {
            return $nrmarcacaoponto;
        } else {
            return false;
        }
    }


    // Dia apuração é importante pois o horário retornado corresponde a jornada de trabalho do dia específico
    public function getDadosHorarioVinculo(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto = null
    )  {

        try {
            if ($dtapuracaoponto === null) {
                $dtapuracaoponto = new \DateTime();
                $dtapuracaoponto = $dtapuracaoponto->format('d/m/Y');
            }

            $vinculom = $this->getVinculom($nrvinculom, $nrorg);

            if (empty($vinculom)) {
                throw new \Exception(constant('messagesMarcacaoRepository')['ERRO_VINCULO_NAO_ENCONTRADO']);
            }

            $nrtipovinculom = $vinculom['NRTIPOVINCULOM'];
            $nrvinculoempreg = $vinculom['NRVINCULOEMPREG'];

            $whereHrprojetarintervalo = "";
            if ($nrtipovinculom == self::TIPO_VINC_CLT && $nrvinculoempreg != self::VINC_EMPREG_APRENDIZ) {
                $whereHrprojetarintervalo = <<<SQL
                    (IH.HRPROJETARINTERVALO IS NOT NULL OR IDTIPOHORARIO = 'F') AND
SQL;
            }

            $sql = <<<SQL
                SELECT
                    HT.NRHORDIAM,
                    HD.IDTIPOHORARIO,
                    IH.NRINTERVALOHORARIO,
                    NVL(IV.HRPROJETARINTERVALO, '0000') HRPROJETARINTERVALO,
                    HH.NMHORDIAH,
                    AE.NRESCALATRABM,
                    EH.QTHRESCALATRABH AS QTJORNADADIARIA
                FROM
                    (
                        SELECT
                            NRORG,
                            MAX(NRTURNO) NRTURNO,
                            NRESCALATRABM
                        FROM
                            GPE_ALTEESCALA
                        WHERE
                            NRORG = :P_NRORG
                            AND
                            NRVINCULOM = :P_NRVINCULOM
                            AND
                            DTINIESCALA = (
                                SELECT MAX(DTINIESCALA)
                                FROM GPE_ALTEESCALA ALT
                                WHERE
                                    ALT.NRORG = GPE_ALTEESCALA.NRORG
                                    AND
                                    ALT.NRVINCULOM = GPE_ALTEESCALA.NRVINCULOM
                                    AND
                                    ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
                        GROUP BY
                            NRORG,
                            NRESCALATRABM
                    ) AE,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - DTDATABASE AS NRDIASENTRE
                        FROM GPE_TURNO
                        WHERE
                            NRORG = :P_NRORG
                    ) TU,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            NRHORDIAM,
                            ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
                        FROM
                            GPE_HORARIOTURNO
                        WHERE
                            NRORG = :P_NRORG
                        ORDER BY
                            NRORG,
                            NRTURNO,
                            NRSEQHORARIO
                    ) HT,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            COUNT(*) AS QTHORARIOS
                        FROM
                            GPE_HORARIOTURNO
                        WHERE
                            NRORG = :P_NRORG
                        GROUP BY
                            NRORG,
                            NRTURNO
                    ) QH,
                    (
                        SELECT
                            NRORG,
                            NRHORDIAM,
                            MAX(DTMESCOMPETENC) AS DTMESCOMPETENC,
                            NMHORDIAH
                        FROM GPE_HORDIAH
                        WHERE
                            NRORG = :P_NRORG
                            AND
                            DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                        GROUP BY
                            NRORG,
                            NRHORDIAM,
                            NMHORDIAH
                    ) HH,
                    (
                        SELECT
                            NRORG,
                            NRHORDIAM,
                            MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO,
                            HRPROJETARINTERVALO
                        FROM
                            GPE_INTERVALOSHORARIO
                        WHERE
                            NRORG = :P_NRORG
                        GROUP BY
                            NRORG,
                            NRHORDIAM,
                            HRPROJETARINTERVALO
                    ) IV,
                    GPE_INTERVALOSHORARIO IH,
                    GPE_HORDIAH HD,
                    GPE_ESCALATRABH EH
                WHERE
                    AE.NRORG = TU.NRORG AND
                    AE.NRTURNO = TU.NRTURNO AND
                    AE.NRORG = HT.NRORG AND
                    AE.NRTURNO = HT.NRTURNO AND
                    HT.NRORG = HH.NRORG AND
                    HT.NRHORDIAM = HH.NRHORDIAM AND
                    HD.NRORG = HH.NRORG AND
                    HD.NRHORDIAM = HH.NRHORDIAM AND
                    HD.DTMESCOMPETENC = HH.DTMESCOMPETENC AND
                    IV.NRORG(+) = HH.NRORG AND
                    IV.NRHORDIAM(+) = HH.NRHORDIAM AND
                    IH.NRORG(+) = IV.NRORG AND
                    IH.NRHORDIAM(+) = IV.NRHORDIAM AND
                    IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO AND
                    AE.NRORG = QH.NRORG AND
                    AE.NRTURNO = QH.NRTURNO AND
                    HT.NRSEQHR - 1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS) AND
                    $whereHrprojetarintervalo
                    EH.NRESCALATRABM = AE.NRESCALATRABM AND
                    EH.NRORG = AE.NRORG AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESCALATRABM = AE.NRESCALATRABM AND
                            DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"            => $nrorg,
                    "P_NRVINCULOM"       => $nrvinculom,
                    "P_DTMESCOMPETENCIA" => DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO),
                    "P_DTAPURACAOPONTO"  => $dtapuracaoponto
                )
            );

            $dadosHorario = $query->fetchAll();

            if (isset($dadosHorario[0])) {
                return $dadosHorario[0];
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getVinculom(
        $nrvinculom,
        $nrorg)
    {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    GPE_VINCULOM
                WHERE
                    NRORG = :P_NRORG
                    AND NRVINCULOM = :P_NRVINCULOM
SQL;

            $params = array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom
            );

            $result = $this->connection->fetchAll($sql, $params);

            return !empty($result) ? $result[0] : $result;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParamorg($nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    GPE_PARAMORG
                WHERE
                    NRORG = :P_NRORG
SQL;

            $params = array(
                'P_NRORG' => $nrorg
            );

            $result = $this->connection->fetchAll($sql, $params);

            return !empty($result) ? $result[0] : $result;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParesMarcacoesDia(array $params) : array {
        $sql = <<<SQL
            SELECT
              ROWNUM,
              NRMARCACAOPONTO,
              MARC.NRVINCULOM,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS DTHRAJUSTADA,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
              DTHRAJUSTADA AS DTHRORDER,
              IDMARCDESPREZADA,
              IDDIAOCORRENCIA,
              DSJUSTIFICATIVA,
              COL.NRCOLETOR,
              COL.DSCOLETOR,
              IDREFERINTERVALO
            FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
               ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND MARC.NRVINCULOM = :P_NRVINCULO
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA = 'DC')
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                   OR (IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY DTHRORDER, NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $paresMarcacoes = $std->fetchAll();

        return $paresMarcacoes;
    }
    
    public function getMarcacaoByNrmarcacaoponto(
        int $nrorg,
        int $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            SELECT
              MP.*,
              TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY') DIA,
              TO_CHAR(MP.DTHRMARCACAOPONTO, 'HH24:MI') HORA_MINUTO
            FROM GPE_MARCACAOPONTO MP
            WHERE MP.NRORG = :P_NRORG
                AND MP.NRMARCACAOPONTO = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }

    public function removeParMarcacao(array $params) : int {
        $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
            WHERE NRORG             = :P_NRORG
              AND NRVINCULOM        = :P_NRVINCULOM
              AND NRMARCACAOPONTO   IN (:P_NRMARCACAOENTRADA, :P_NRMARCACAOSAIDA)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacaoesRemovidas = $std->rowCount();

        return $marcacaoesRemovidas;
    }
    
    public function removeMarcacao(array $params) : int {
        $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
            WHERE NRORG             = :P_NRORG
              AND NRVINCULOM        = :P_NRVINCULOM
              AND NRMARCACAOPONTO   = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacaoesRemovidas = $std->rowCount();

        return $marcacaoesRemovidas;
    }

    public function saveMarcacao(array $params) : bool {
        $params['P_NRMARCACAO'] = isset($params['P_NRMARCACAO']) && isset($params['P_NRORG']) ? $params['P_NRMARCACAO'] : NovoCodigo::geraCodigo($params['P_NRORG'], 'GPE_MARCACAOPONTO', 1, 22, false);
        
        $sql = <<<SQL
            INSERT INTO GPE_MARCACAOPONTO (
                      NRMARCACAOPONTO,
                      NRVINCULOM,
                      DTHRMARCACAOPONTO,
                      DTHRAJUSTADA,
                      IDENQUADRAHORARIO,
                      IDMARCDESPREZADA,
                      IDDIAOCORRENCIA,
                      DSJUSTIFICATIVA,
                      NRORGINCLUSAO,
                      NRCOLETOR,
                      IDREFERINTERVALO,
                      IDATIVO,
                      DTINCLUSAO,
                      CDOPERINCLUSAO,
                      NRORG
                  ) VALUES (
                      :P_NRMARCACAO,
                      :P_NRVINCULOM,
                      TO_DATE(:P_DTHRMARCACAO, 'DD/MM/YYYY HH24:MI:SS'),
                      TO_DATE(:P_DTHRMARCACAO, 'DD/MM/YYYY HH24:MI:SS'),
                      'N',
                      'N',
                      :P_IDDIAOCORRENCIA,
                      '',
                      '',
                      24,
                      'N',
                      'S',
                      SYSDATE,
                      1,
                      :P_NRORG)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function getMarcacoesDia(array $params): array {
        $sql = <<<SQL
            SELECT MP.*,
                   TO_CHAR(DTHRAJUSTADA, 'HH24:MI') HORA_MINUTO
            FROM GPE_MARCACAOPONTO MP
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULO
              AND ((IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') + 1)
                OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') - 1)
                OR (IDDIAOCORRENCIA IN ('FC', 'DC') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY')))
            ORDER BY DTHRAJUSTADA ASC, NRMARCACAOPONTO ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoes = $std->fetchAll();
        
        return $marcacoes;
    }
    
    public function getPrimeiraAusenciaMarcacaoDia(
        string $nrorg,
        string $nrvinculom,
        string $date
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                (
                    SELECT  
                        ROWNUM RN,
                        MARCACAOPONTO.* 
                    FROM (
                        SELECT
                            *
                        FROM
                            GPE_MARCACAOPONTO MP
                        WHERE
                            NRORG = :P_NRORG
                            AND NRVINCULOM = :P_NRVINCULOM
                            AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                            AND NRCOLETOR <> 21 /* BALIZA */
                            
                        ORDER BY
                            DTHRMARCACAOPONTO
                    ) MARCACAOPONTO
                    
                    WHERE
                        (
                            SELECT
                                COUNT(*) ROWCOUNT
                            FROM
                            GPE_MARCACAOPONTO MP
                            WHERE
                                NRORG = :P_NRORG
                                AND NRVINCULOM = :P_NRVINCULOM
                                AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                                AND NRCOLETOR <> 21 /* BALIZA */
                        ) > 2 /* GARANTE QUE TENHA PELO MENOS UM INTERVALO */
                )
                
            WHERE
                RN = 2 /* PRIMERIA AUSÊNCIA */
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'               => $nrorg,
                'P_NRVINCULOM'          => $nrvinculom,
                'P_DTHRMARCACAOPONTO'   => $date
            )    
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }
    
    public function getMarcacaoDiaByOrdem(
        string  $nrorg,
        string  $nrvinculom,
        string  $date,
        int     $posicaoMarcacaoDia = 1,
        string  $ordem              = Utils::ORDEM_ASC
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                (
                    SELECT  
                        ROWNUM RN,
                        MARCACAOPONTO.* 
                    FROM (
                        SELECT
                            *
                        FROM
                            GPE_MARCACAOPONTO MP
                        WHERE
                            NRORG = :P_NRORG
                            AND NRVINCULOM = :P_NRVINCULOM
                            AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                            AND NRCOLETOR <> 21 /* BALIZA */
                            
                        ORDER BY
                            DTHRMARCACAOPONTO $ordem
                    ) MARCACAOPONTO
                )
                
            WHERE
                RN = :P_POSICAO_MARCACAO_DIA /* SELECIONA A MARCAÇÃO SEQUENCIAL */
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'                   => $nrorg,
                'P_NRVINCULOM'              => $nrvinculom,
                'P_DTHRMARCACAOPONTO'       => $date,
                'P_POSICAO_MARCACAO_DIA'    => $posicaoMarcacaoDia,
            )    
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }
    
    public function getMarcacaoDiaEntrada(
        string  $nrorg,
        string  $nrvinculom,
        string  $date
    ) {
        
        return $this->getMarcacaoDiaByOrdem(
            $nrorg,
            $nrvinculom,
            $date,
            MarcacaoService::POSICAO_MARCACAO_ENTRADA,
            MarcacaoService::ORDEM_MARCACAO_ENTRADA
        );
    }
    
    public function getMarcacaoDiaSaida(
        string  $nrorg,
        string  $nrvinculom,
        string  $date
    ) {
        
        return $this->getMarcacaoDiaByOrdem(
            $nrorg,
            $nrvinculom,
            $date,
            MarcacaoService::POSICAO_MARCACAO_SAIDA,
            MarcacaoService::ORDEM_MARCACAO_SAIDA
        );
    }
    
    public function getMarcacaoByDataMarcacao(
        $nrorg,
        $nrvinculom,
        $horaMarcacao,
        $diaMarcacao,
        bool $all = false
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_MARCACAOPONTO MP
            WHERE
                MP.NRORG                                        = :P_NRORG
                AND MP.NRVINCULOM                               = :P_NRVINCULOM
                AND TO_CHAR(mp.dthrmarcacaoponto, 'HH24:MI')    = :P_HORAMARCACAO
                AND TO_CHAR(mp.dthrmarcacaoponto, 'DD/MM/YYYY') = :P_DIAMARCACAO
                AND MP.NRCOLETOR                                <> 21
        
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
                'P_HORAMARCACAO'    => $horaMarcacao,
                'P_DIAMARCACAO'     => $diaMarcacao,
            )    
        );

        $marcacao = $all ?
            $std->fetchAll() :
            $std->fetch();
        
        return $marcacao;
    }
    
    public function getMarcacaoByDiaMarcacao(
        $nrorg,
        $nrvinculom,
        $diaMarcacao
    ) {
        
        if (!is_array($diaMarcacao)) {
            $paramDiaMarcacao = array($diaMarcacao);
        } else {
            $paramDiaMarcacao = $diaMarcacao;
        }
        
        $paramDiaMarcacao = '\'' . implode('\', \'', $paramDiaMarcacao) . '\'';
        
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_MARCACAOPONTO MP
            WHERE
                MP.NRORG                                        = :P_NRORG
                AND MP.NRVINCULOM                               = :P_NRVINCULOM
                AND TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY') IN ($paramDiaMarcacao)
                AND MP.NRCOLETOR                                <> 21
        
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
            )    
        );

        $marcacao = $std->fetchAll();
        
        return $marcacao;
    }
    
    public function limpaIdFixaMarcacaoDia(
        $nrOrg,
        $nrVinculom,
        $dia,
        $idFixaEntradaSaida
    ) {
        
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND IDFIXAENTSAID            = :P_IDFIXAENTSAID
SQL;

        $std = $this->connection->prepare($sql);

        $result = $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTAPURACAOPONTO" => $dia,
                "P_IDFIXAENTSAID"   => $idFixaEntradaSaida
            )
        );
        
        return $result;
    }
    
    public function limpaIdFixaMarcacaoDiaEntrada(
        $nrOrg,
        $nrVinculom,
        $dia
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_ENTRADA;
        
        return $this->limpaIdFixaMarcacaoDia(
            $nrOrg,
            $nrVinculom,
            $dia,
            $idFixaEntradaSaida
        );
    }
    
    public function limpaIdFixaMarcacaoDiaSaida(
        $nrOrg,
        $nrVinculom,
        $dia
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_SAIDA;
        
        return $this->limpaIdFixaMarcacaoDia(
            $nrOrg,
            $nrVinculom,
            $dia,
            $idFixaEntradaSaida
        );
    }
    
    public function setIdFixaMarcacao(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto,
        $idFixaEntSaid
    ) {
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = :P_IDFIXAENTSAID
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRMARCACAOPONTO          = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);

        $result = $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTAPURACAOPONTO" => $dtApuracaoPonto,
                "P_NRMARCACAOPONTO" => $nrMarcacaoPonto,
                "P_IDFIXAENTSAID"   => $idFixaEntSaid
            )
        );
        
        return $result;
    }
    
    public function setIdFixaMarcacaoSaida(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_SAIDA;
        
        return $this->setIdFixaMarcacao(
            $nrOrg,
            $nrVinculom,
            $dtApuracaoPonto,
            $nrMarcacaoPonto,
            $idFixaEntradaSaida
        );
    }
    
    public function setIdFixaMarcacaoEntrada(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_ENTRADA;
        
        return $this->setIdFixaMarcacao(
            $nrOrg,
            $nrVinculom,
            $dtApuracaoPonto,
            $nrMarcacaoPonto,
            $idFixaEntradaSaida
        );
    }
    
    public function getMarcacoesPontoDiaOpcoesFixarMarcacao(
        array $params
    ): array {

        $sql = <<<SQL
            SELECT
              NRMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS NRMARCACAOPONTO_LABEL,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO
             FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
              ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND NRVINCULOM = :P_NRVINCULOM
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA IN ('DA', 'DC', 'DS'))
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                  OR (IDDIAOCORRENCIA = 'FA' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                  OR (IDDIAOCORRENCIA = 'FS' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                  OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY NRMARCACAOPONTO_LABEL, NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoesPonto = $std->fetchAll();

        return $marcacoesPonto;
    }
    
    public function getParMarcacaoAlmoco(
        $nrOrg,
        $nrVinculom,
        $dtMarcacaoPonto    
    ) {

        $sql = <<<SQL
            SELECT
                MARCACAO_ALMOCO.DTHRMARCACAOPONTO       AS DTHRMARCACAOPONTO_ALMOCO,
                MARCACAO_ALMOCO.NRMARCACAOPONTO         AS NRMARCACAOPONTO_ALMOCO,
                MARCACAO_POS_ALMOCO.DTHRMARCACAOPONTO   AS DTHRMARCACAOPONTO_POS_ALMOCO,
                MARCACAO_POS_ALMOCO.NRMARCACAOPONTO     AS NRMARCACAOPONTO_POS_ALMOCO
            FROM
                (
                    SELECT
                        MP.*
                    FROM
                        PTO_PROGOCORRENCIA   PO
                        JOIN GPE_MARCACAOPONTO    MP ON PO.NRORG = MP.NRORG
                                                     AND PO.NRMARCACAOPONTO = MP.NRMARCACAOPONTO
                    WHERE
                        PO.NRORG = :P_NRORG
                        AND PO.NRVINCULOM = :P_NRVINCULOM
                        AND TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') = :P_DTMARCACAOPONTO
                        AND PO.NRTIPOPROGOCORRENCIA IN (20, 26) /* ALMOÇO */
                ) MARCACAO_ALMOCO
            JOIN (
                    SELECT
                        *
                    FROM
                        (
                            SELECT
                                *
                            FROM
                                GPE_MARCACAOPONTO MP2
                    WHERE
                                NRORG = :P_NRORG
                                AND NRVINCULOM = :P_NRVINCULOM
                                AND DTHRMARCACAOPONTO > (
                                    SELECT
                                        DTHRMARCACAOPONTO
                                    FROM
                                        (
                                            SELECT
                                                MP.*
                                            FROM
                                                PTO_PROGOCORRENCIA   PO
                                                JOIN GPE_MARCACAOPONTO    MP ON PO.NRORG = MP.NRORG
                                                                             AND PO.NRMARCACAOPONTO = MP.NRMARCACAOPONTO
                                    WHERE
                                            PO.NRORG = :P_NRORG
                                            AND PO.NRVINCULOM = :P_NRVINCULOM
                                            AND TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') = :P_DTMARCACAOPONTO
                                            AND PO.NRTIPOPROGOCORRENCIA IN (20, 26)
                                            AND ROWNUM = 1
                                    )
                                )
                            ORDER BY
                                MP2.DTHRMARCACAOPONTO
                        )
                    WHERE
                        ROWNUM = 1
                ) MARCACAO_POS_ALMOCO ON MARCACAO_POS_ALMOCO.NRORG = MARCACAO_ALMOCO.NRORG
                                         AND MARCACAO_POS_ALMOCO.NRVINCULOM = MARCACAO_ALMOCO.NRVINCULOM
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTMARCACAOPONTO" => $dtMarcacaoPonto
            )
        );

        $marcacoesPonto = $std->fetch();

        return $marcacoesPonto;

    }

    public function reverterEntradaSaida(
        array $params
    ) {
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DATE, 'DD/MM/YYYY')
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function removeMarcacoesDia(
        array $params
    ) {
        $sql = <<<SQL
            DELETE FROM
                GPE_MARCACAOPONTO
            WHERE
                NRORG = :P_NRORG AND
                NRVINCULOM = :P_NRVINCULO AND
                TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY') = :P_DATE
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getDetalheAcompanhamento(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                TO_CHAR(MARC.DTHRAJUSTADA, 'YYYY/MM/DD') DATE_ORDER,
                TO_CHAR(MARC.DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
                TO_CHAR(MARC.DTHRAJUSTADA, 'DD/MM/YYYY') AS DTAPURACAOSITUFREQ,
                MARC.NRMARCACAOPONTO,
                MARC.IDDIAOCORRENCIA
            FROM
                GPE_MARCACAOPONTO MARC
            WHERE
                MARC.NRORG = :P_NRORG
                AND MARC.NRCOLETOR          != 21
                AND MARC.NRCOLETOR          != 25
                AND MARC.NRVINCULOM         = :P_NRVINCULOM
                AND INSTR(
                        :P_DATES,
                        ',' ||
                        (
                            CASE
                                WHEN MARC.IDDIAOCORRENCIA IN ('DS', 'FS') THEN TO_CHAR(MARC.DTHRAJUSTADA + 1, 'DD/MM/YYYY')
                                WHEN MARC.IDDIAOCORRENCIA IN ('DA', 'FA') THEN TO_CHAR(MARC.DTHRAJUSTADA - 1, 'DD/MM/YYYY')
                                ELSE TO_CHAR(MARC.DTHRAJUSTADA, 'DD/MM/YYYY')
                            END
                        ) ||
                        ','
                    ) > 0
            ORDER BY MARC.DTHRAJUSTADA ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoes = $std->fetchAll();

        return $marcacoes;
    }
}