<?php

namespace Ems\Services;

use Util\Environment;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Controller\Report;
use Zeedhi\Framework\Report\ReportService as ZhReportService;
use Ems\Util\FileServerClasses\Util\MyCurl;
use Ems\Util\Utilities;

class ReportService {
    /** @var Environment */
    protected $environment;

    /** @var ZhReportService */
    protected $zhReportService;

    public function __construct(
        Environment $environment,
        ZhReportService $zhReportService
    ) {
        $this->environment = $environment;
        $this->zhReportService = $zhReportService;
    }

    public function generateBirtReport(
        array $row
    ): string {
        $strategy = 'BIRT';
        $reportName = (string) $row['reportName'];

        $rowCopy = (new \ArrayObject($row))->getArrayCopy();

        unset($rowCopy['reportName']);
        $rowCopy = $this->setDefaultParameters($rowCopy);
        //$rowCopy = $this->setRowByReportName($rowCopy, $reportName);
        $link = $this->zhReportService->createRemoteReport(
            $strategy, $reportName, $rowCopy
        );

        return $link;
    }

    public function setDefaultParameters(
        array $row
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $row['P_NRORG']          = $nrOrg;
        $row['P_NRORG_PADRAO']   = $nrOrgPadrao;

        return $row;
    }

    private function setRowByReportName(
        array $row,
        string $reportName
    ): array {
        if ($reportName === 'folha_ponto') {
            return $this->setFolhaPontoRow($row);
        }

        throw new \Exception("O  relatório {$reportName}  não existe!");
        return $row;
    }

    private function setFolhaPontoRow(
        array $row
    ): array {
        $nrVinculo = (int) $row['NRVINCULOM'];
        $dtCompetencia = (string) $row['DTMESCOMPETENCIA'];

        unset($row['NRVINCULOM']);
        $row['P_NRVINCULOM'] = $nrVinculo;

        unset($row['DTMESCOMPETENCIA']);
        $row['P_COMPETENCIA'] = $dtCompetencia;

        $row['P_COMPPONTO'] = '';

        return $row;
    }

    public function downloadReport(
        string $link
    ): array {
        $reportBlob = MyCurl::get($link, [], false);
        $file = Utilities::blobToBase64($reportBlob);

        return $file;
    }
}