<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil };

class SolicitacaoService {

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var Environment */
    protected $environment;

    /** @var SolicitacaoRepository */
    protected $solicitacaoRepository;

    public function __construct(
        SolicitacaoRepository   $solicitacaoRepository,
        Environment             $environment,
        EstruturaService        $estruturaService
    ) {
        $this->solicitacaoRepository    = $solicitacaoRepository;
        $this->environment              = $environment;
        $this->estruturaService         = $estruturaService;
    }

    public function getSolicitacoesVinculosGestor(
        int $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO);

        $nrEstruturas = $this->estruturaService->
            getEstruturasInferiores($nrEstrutura, true);

        $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRESTRUTURAS'    => $nrEstruturasString,
            'P_DTMESCOMPETENC'  => $dtmescompetenc
        ];

        $solicitacoes = $this->solicitacaoRepository->
            getSolicitacoesVinculosGestor($params);

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getItemSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getTiposolicitacao() {
        $nrorg = $this->environment->getNrorg();
        $tipos = $this->solicitacaoRepository->getTiposolicitacao($nrorg);

        foreach($tipos as &$tipo) {
            if ($tipo['NRTIPOSOLICITACAO'] === '6') {
                $tipo['NMTIPOSOLICITACAO'] = 'Solicitação de Seleção de Pessoal';
            }
        }

        return $tipos;
    }

    public function getTipoRelacionamento() {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposRel = $this->solicitacaoRepository->getTipoRelacionamento(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('1/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
        ];

        $linhas = $this->solicitacaoRepository->getLinhaTransporteAtiva(
            $params
        );

        return $linhas;
    }

    public function getTipolinhavinc(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposLinhas = $this->solicitacaoRepository->getTipolinhavinc(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposLinhas;
    }

    public function getIntegtransp(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORG_PADRAO'        => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
        ];

        $numerosIntegracao = $this->solicitacaoRepository->getIntegtransp(
            $params
        );

        return $numerosIntegracao;
    }

    public function getSentidolinha(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $numerosIntegracao = $this->solicitacaoRepository->getSentidolinha(
            $nrOrg, $nrOrgPadrao
        );

        return $numerosIntegracao;
    }

    public function getBeneficio(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $beneficios = $this->solicitacaoRepository->getBeneficio(
            $nrOrg, $nrOrgPadrao
        );

        return $beneficios;
    }

    public function getPessoaRelacionamentos(
        int $nrPessoa
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORGPADRAO'         => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
            'P_NRPESSOA'            => $nrPessoa,
            'P_NRTIPORELACIONA'     => null,
        ];

        $pessoas = $this->solicitacaoRepository->getPessoaRelacionamentos(
            $params
        );

        return $pessoas;
    }

    public function getTipoDependente(): array {
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();

        $tiposDep = $this->solicitacaoRepository->getTipoDependente($nrOrgPadrao);

        return $tiposDep;
    }

    public function getTemplate(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $templates = $this->solicitacaoRepository->getTemplate($nrOrg);

        return $templates;
    }

    public function getTipoprocessos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getTipoprocessos($nrOrg);

        return $tipos;
    }

    public function getParcnegocioVinculos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getParcnegocioVinculos($nrOrg);

        return $tipos;
    }

    public function getTipoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->solicitacaoRepository->getTipoTransferencia(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getMovimentacaoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
            'P_DTMESCOMPETENC' => $dtMesCompetencia,
        ];

        $tipos = $this->solicitacaoRepository->getMovimentacaoTransferencia(
            $params
        );

        return $tipos;
    }

    public function getMotivoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $motivos = $this->solicitacaoRepository->getMotivoTransferencia(
            $nrOrg
        );

        return $motivos;
    }
}
