<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 30/05/2018
 * Time: 12:40
 */

namespace Service\Hcm;

class GeralService {

    private $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getPaisByNome($nmpais){
        try {
            $query = <<<SQL
SELECT *
  FROM PAIS
 WHERE UPPER(NMPAIS) = UPPER(:P_NMPAIS)
SQL;
            $params = array(
                'P_NMPAIS' => $nmpais,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMunicipioByPaisAndEstadoAndNome($cdpais, $sgestado, $nmmunicipio){
        try {
            $query = <<<SQL
SELECT *
  FROM MUNICIPIO
 WHERE CDPAIS = :P_CDPAIS
   AND SGESTADO = :P_SGESTADO
   AND UPPER(NMMUNICIPIO) = UPPER(:P_NMMUNICIPIO)
SQL;
            $params = array(
                'P_CDPAIS' => $cdpais,
                'P_SGESTADO' => $sgestado,
                'P_NMMUNICIPIO' => $nmmunicipio,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getBairroByPaisAndEstadoAndMunicipioAndNome($cdpais, $sgestado, $cdmunicipio, $nmbairro){
        try {
            $query = <<<SQL
SELECT *
  FROM BAIRRO
 WHERE CDPAIS = :P_CDPAIS
   AND SGESTADO = :P_SGESTADO
   AND CDMUNICIPIO = :P_CDMUNICIPIO
   AND UPPER(NMBAIRRO) = UPPER(:P_NMBAIRRO)
SQL;
            $params = array(
                'P_CDPAIS' => $cdpais,
                'P_SGESTADO' => $sgestado,
                'P_CDMUNICIPIO' => $cdmunicipio,
                'P_NMBAIRRO' => $nmbairro,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function addParceiroNegocio($nrorg, $nrparcnegocio, $nmprincipalparc, $nmsecundariparc, $idpessoafisica, $idinstituicao, $idparcfundido, $dtnascifundparc, 
                                       $nrinscricaoparc, $cdtipoparcprincipal, $cdtipoinscricao, $cdoperinclusao){
        try {
            $sql = <<<SQL
INSERT INTO PARCNEGOCIO
    (NRORG, NRPARCNEGOCIO, NMPRINCIPALPARC, NMSECUNDARIPARC, IDPESSOAFISICA, IDINSTITUICAO, IDPARCFUNDIDO, DTNASCIFUNDPARC, NRINSCRICAOPARC, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
VALUES 
    (:NRORG, :NRPARCNEGOCIO, :NMPRINCIPALPARC, :NMSECUNDARIPARC, :IDPESSOAFISICA, :IDINSTITUICAO, :IDPARCFUNDIDO, TO_DATE(:DTNASCIFUNDPARC,'DD/MM/YYYY'), :NRINSCRICAOPARC, :CDTIPOPARCPRINCIPAL, :CDTIPOINSCRICAO, 'S', SYSDATE, 1, :CDOPERINCLUSAO)
SQL;
            $params = array(
                'NRORG' => $nrorg,
                'NRPARCNEGOCIO' => $nrparcnegocio,
                'NMPRINCIPALPARC' => $nmprincipalparc,
                'NMSECUNDARIPARC' => $nmsecundariparc,
                'IDPESSOAFISICA' => $idpessoafisica,
                'IDINSTITUICAO' => $idinstituicao,
                'IDPARCFUNDIDO' => $idparcfundido,
                'DTNASCIFUNDPARC' => $dtnascifundparc,
                'NRINSCRICAOPARC' => $nrinscricaoparc,
                'CDTIPOPARCPRINCIPAL' => $cdtipoparcprincipal,
                'CDTIPOINSCRICAO' => $cdtipoinscricao,
                'CDOPERINCLUSAO' => $cdoperinclusao,
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function addEnderecoParceiroNegocio($nrorg, $nrenderecoparc, $nrparcnegocio, $cdtipoendereco, $cdpais, $sgestado, $cdmunicipio, $cdbairro, $cdlogradouro, 
                                               $nmbairroendereco, $dscompleendereco, $nrcependereco, $dsreferenciaende, $dsendereco, $nrimovelendereco, $cdoperinclusao){
        try {
            $sql = <<<SQL
INSERT INTO ENDERECOPARC
    (NRORG, NRENDERECOPARC, NRPARCNEGOCIO, CDTIPOENDERECO, CDPAIS, SGESTADO, CDMUNICIPIO, CDBAIRRO, CDLOGRADOURO, NMBAIRROENDERECO, DSCOMPLEENDERECO, NRCEPENDERECO, DSREFERENCIAENDE, DSENDERECO, NRIMOVELENDERECO, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
VALUES 
    (:NRORG, :NRENDERECOPARC, :NRPARCNEGOCIO, :CDTIPOENDERECO, :CDPAIS, :SGESTADO, :CDMUNICIPIO, :CDBAIRRO, :CDLOGRADOURO, :NMBAIRROENDERECO, :DSCOMPLEENDERECO, :NRCEPENDERECO, :DSREFERENCIAENDE, :DSENDERECO, :NRIMOVELENDERECO, 'S', SYSDATE, 1, :CDOPERINCLUSAO)
SQL;
            $params = array(
                'NRORG' => $nrorg,
                'NRENDERECOPARC' => $nrenderecoparc,
                'NRPARCNEGOCIO' => $nrparcnegocio,
                'CDTIPOENDERECO' => $cdtipoendereco,
                'CDPAIS' => $cdpais,
                'SGESTADO' => $sgestado,
                'CDMUNICIPIO' => $cdmunicipio,
                'CDBAIRRO' => $cdbairro,
                'CDLOGRADOURO' => $cdlogradouro,
                'NMBAIRROENDERECO' => $nmbairroendereco,
                'DSCOMPLEENDERECO' => $dscompleendereco,
                'NRCEPENDERECO' => $nrcependereco,
                'DSREFERENCIAENDE' => $dsreferenciaende,
                'DSENDERECO' => $dsendereco,
                'NRIMOVELENDERECO' => $nrimovelendereco,
                'CDOPERINCLUSAO' => $cdoperinclusao,
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function getParametrosOrg($nrorg){
        try {
            $query = <<<SQL
SELECT * FROM GPE_PARAMORG WHERE NRORG = :P_NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
            );
            
            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            
            return $result;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}